# Print an optspec for argparse to handle cmd's options that are independent of any subcommand.
function __fish_overmask_global_optspecs
	string join \n s/seed-file= o/overlay-file= m/mask-file= b/block-size= i/ignore-errors h/help V/version
end

function __fish_overmask_needs_command
	# Figure out if the current invocation already has a command.
	set -l cmd (commandline -opc)
	set -e cmd[1]
	argparse -s (__fish_overmask_global_optspecs) -- $cmd 2>/dev/null
	or return
	if set -q argv[1]
		# Also print the command, so this can be used to figure out what it is.
		echo $argv[1]
		return 1
	end
	return 0
end

function __fish_overmask_using_subcommand
	set -l cmd (__fish_overmask_needs_command)
	test -z "$cmd"
	and return 1
	contains -- $cmd[1] $argv
end

complete -c overmask -n "__fish_overmask_needs_command" -s s -l seed-file -d 'Where original read-only data should be read from' -r -F
complete -c overmask -n "__fish_overmask_needs_command" -s o -l overlay-file -d 'Where modified (written) data should be stored' -r -F
complete -c overmask -n "__fish_overmask_needs_command" -s m -l mask-file -d 'Where a mask of the modified data should be stored' -r -F
complete -c overmask -n "__fish_overmask_needs_command" -s b -l block-size -d 'Block size for all read and write operations' -r
complete -c overmask -n "__fish_overmask_needs_command" -s i -l ignore-errors -d 'Ignore IO errors from the underlying files'
complete -c overmask -n "__fish_overmask_needs_command" -s h -l help -d 'Print help'
complete -c overmask -n "__fish_overmask_needs_command" -s V -l version -d 'Print version'
complete -c overmask -n "__fish_overmask_needs_command" -f -a "apply" -d 'Apply the overlay on top of the seed using the mask'
complete -c overmask -n "__fish_overmask_needs_command" -f -a "a" -d 'Apply the overlay on top of the seed using the mask'
complete -c overmask -n "__fish_overmask_needs_command" -f -a "clean" -d 'Deduplicate data between the seed and overlay'
complete -c overmask -n "__fish_overmask_needs_command" -f -a "c" -d 'Deduplicate data between the seed and overlay'
complete -c overmask -n "__fish_overmask_needs_command" -f -a "device" -d 'Create a virtual block device to capture writes'
complete -c overmask -n "__fish_overmask_needs_command" -f -a "d" -d 'Create a virtual block device to capture writes'
complete -c overmask -n "__fish_overmask_needs_command" -f -a "dev" -d 'Create a virtual block device to capture writes'
complete -c overmask -n "__fish_overmask_needs_command" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c overmask -n "__fish_overmask_using_subcommand apply" -l force
complete -c overmask -n "__fish_overmask_using_subcommand apply" -s h -l help -d 'Print help'
complete -c overmask -n "__fish_overmask_using_subcommand a" -l force
complete -c overmask -n "__fish_overmask_using_subcommand a" -s h -l help -d 'Print help'
complete -c overmask -n "__fish_overmask_using_subcommand clean" -s t -l truncate -d 'Truncate the overlay and mask files to the last used byte'
complete -c overmask -n "__fish_overmask_using_subcommand clean" -s h -l help -d 'Print help'
complete -c overmask -n "__fish_overmask_using_subcommand c" -s t -l truncate -d 'Truncate the overlay and mask files to the last used byte'
complete -c overmask -n "__fish_overmask_using_subcommand c" -s h -l help -d 'Print help'
complete -c overmask -n "__fish_overmask_using_subcommand device" -s n -l nbd-device -d 'nbd device file (requires nbd kernel module)' -r -F
complete -c overmask -n "__fish_overmask_using_subcommand device" -s t -l nbd-timeout -d 'nbd device timeout in seconds' -r
complete -c overmask -n "__fish_overmask_using_subcommand device" -s p -l print-operations -d 'Print every IO operation (`read()`, `write()`, `flush()`, etc)'
complete -c overmask -n "__fish_overmask_using_subcommand device" -s T -l trim-no-punch-holes -d 'Don\'t punch holes (fallocate) in overlay and mask on `trim()`'
complete -c overmask -n "__fish_overmask_using_subcommand device" -s h -l help -d 'Print help'
complete -c overmask -n "__fish_overmask_using_subcommand d" -s n -l nbd-device -d 'nbd device file (requires nbd kernel module)' -r -F
complete -c overmask -n "__fish_overmask_using_subcommand d" -s t -l nbd-timeout -d 'nbd device timeout in seconds' -r
complete -c overmask -n "__fish_overmask_using_subcommand d" -s p -l print-operations -d 'Print every IO operation (`read()`, `write()`, `flush()`, etc)'
complete -c overmask -n "__fish_overmask_using_subcommand d" -s T -l trim-no-punch-holes -d 'Don\'t punch holes (fallocate) in overlay and mask on `trim()`'
complete -c overmask -n "__fish_overmask_using_subcommand d" -s h -l help -d 'Print help'
complete -c overmask -n "__fish_overmask_using_subcommand dev" -s n -l nbd-device -d 'nbd device file (requires nbd kernel module)' -r -F
complete -c overmask -n "__fish_overmask_using_subcommand dev" -s t -l nbd-timeout -d 'nbd device timeout in seconds' -r
complete -c overmask -n "__fish_overmask_using_subcommand dev" -s p -l print-operations -d 'Print every IO operation (`read()`, `write()`, `flush()`, etc)'
complete -c overmask -n "__fish_overmask_using_subcommand dev" -s T -l trim-no-punch-holes -d 'Don\'t punch holes (fallocate) in overlay and mask on `trim()`'
complete -c overmask -n "__fish_overmask_using_subcommand dev" -s h -l help -d 'Print help'
complete -c overmask -n "__fish_overmask_using_subcommand help; and not __fish_seen_subcommand_from apply clean device help" -f -a "apply" -d 'Apply the overlay on top of the seed using the mask'
complete -c overmask -n "__fish_overmask_using_subcommand help; and not __fish_seen_subcommand_from apply clean device help" -f -a "clean" -d 'Deduplicate data between the seed and overlay'
complete -c overmask -n "__fish_overmask_using_subcommand help; and not __fish_seen_subcommand_from apply clean device help" -f -a "device" -d 'Create a virtual block device to capture writes'
complete -c overmask -n "__fish_overmask_using_subcommand help; and not __fish_seen_subcommand_from apply clean device help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
